import {
  ColumnDirective,
  ColumnsDirective,
  Inject,
  Reorder,
  TreeGridComponent,
  Resize,
  Edit,
  Toolbar,
  EditSettings,
} from "@syncfusion/ej2-react-treegrid";
import React, { useEffect, useRef, useState } from "react";
import { summaryData } from "../datasource";
import "./material.css";
import "./TreeGrid.css";

const TreeGrid = () => {
  const [editOptions, setEditOptions] = useState({
    allowAdding: true,
    allowDeleting: true,
    allowEditing: true,
    newRowPosition: "Below",
    // mode: 'Cell'
  });

  const [data, setData] = useState([]);

  const [keys, setKeys] = useState([]);

  const [inputColumn, setInputColumn] = useState("");

  const [expanded, setExpanded] = useState(false);

  const ref = useRef();

  const [toolbarSettings, setToolbarSettings] = useState(["Add"]);

  const [selectedColumns, setSelectedColumns] = useState([]);

  useEffect(() => {
    setData(summaryData);
  }, [summaryData]);

  useEffect(() => {
    if (data.length !== 0) {
      let keysarr = Object.keys(data[0])
        .filter((key) => key !== "category")
        .filter((key) => key !== "subtask");
      let initialArray = ["category"];
      initialArray.push(...keysarr);
      setKeys(initialArray);
    }
  }, [data]);

  useEffect(() => {
    if (!expanded) {
      var elements = document.getElementsByClassName("e-headercelldiv");
      for (var i = 0; i < elements.length; i++) {
        elements[i].classList.add("hide-treegridheader");
        elements[i].classList.remove("show-treegridheader");
      }
    }
    if (expanded) {
      var elements = document.getElementsByClassName("e-headercelldiv");
      for (var i = 0; i < elements.length; i++) {
        elements[i].classList.add("show-treegridheader");
        elements[i].classList.remove("hide-treegridheader");
      }
    }
  }, [expanded]);

  useEffect(() => {
    setTimeout(() => {
      var elements = document.getElementsByClassName("e-headercelldiv");
      for (var i = 0; i < elements.length; i++) {
        elements[i].classList.add("hide-treegridheader");
        elements[i].classList.remove("show-treegridheader");
      }
      var contentelements = document.getElementsByClassName("e-content");
      for (var i = 0; i < contentelements.length; i++) {
        contentelements[i].classList.add("hide-contentoverflow");
      }
     
    }, 100);
  }, []);

  const customizeCell = (args) => {
    const cell = args.cell;
    if (args.column.field !== "category") {
      cell.setAttribute(
        "style",
        "background-color:#F4F5F7;background-clip:content-box;padding: 8px 21px;"
      );
    }
  };

  const addColumn = (inp) => {
    let updated = data.map((obj) => ({ ...obj, [inp]: "" }));
    setData(updated);
    ref.current.refresh();
  };

  const autoFit = () => {
    ref.current.autoFitColumns(["category"]);
  };

  const handleExpand = () => {
    setExpanded(true);
    var elements = document.getElementsByClassName("e-rowcell");
    for (var i = 0; i < elements.length; i++) {
      elements[i].classList.add("show-treegridborder");
      elements[i].classList.remove("hide-treegridborder");
    }
    var contentelements = document.getElementsByClassName("e-content");
    for (var i = 0; i < contentelements.length; i++) {
      contentelements[i].classList.add("show-contentoverflow");
      contentelements[i].classList.remove("hide-contentoverflow");
    }
  };

  const handleCollapse = () => {
    setExpanded(false);
    var elements = document.getElementsByClassName("e-rowcell");
    for (var i = 0; i < elements.length; i++) {
      elements[i].classList.add("hide-treegridborder");
      elements[i].classList.remove("show-treegridborder");
    }
    var contentelements = document.getElementsByClassName("e-content");
    for (var i = 0; i < contentelements.length; i++) {
      contentelements[i].classList.add("hide-contentoverflow");
      contentelements[i].classList.remove("show-contentoverflow");
    }
  };

  const handleColumnSelect = (e) => {
    if (e.target.checked) {
      setSelectedColumns((prevState) => prevState.concat(e.target.name));
    } else {
      setSelectedColumns((prevState) =>
        prevState.filter((item) => item !== e.target.name)
      );
    }
  };

  const handleDeleteRollup = () => {
    for(var j=0;j<data.length;j++){
      for (var i = 0; i < selectedColumns.length; i++) {
        var x= data[j]
        if (x.hasOwnProperty(selectedColumns[i])) {
          delete x[selectedColumns[i]];
        }
        if(x.hasOwnProperty("subtask")){
          for(var k=0;k<x.subtask.length;k++){
            for (var i = 0; i < selectedColumns.length; i++) {
              var y= x.subtask[k]
              if (y.hasOwnProperty(selectedColumns[i])) {
                delete y[selectedColumns[i]];
              }
            }
          }
        }
      }
    }
    ref.current.refresh();
    setTimeout(()=>{
      if (data.length !== 0) {
        let keysarr = Object.keys(data[0])
          .filter((key) => key !== "category")
          .filter((key) => key !== "subtask");
        let initialArray = ["category"];
        initialArray.push(...keysarr);
        setKeys(initialArray);
      }
    },100)
  };


  const headerDiv = (heading, index) => {
    return (
      <div>
        {heading === "category" ? (
          <span>{heading}</span>
        ) : (
          <div>
            <input
              type="checkbox"
              name={`${heading}`}
              value={selectedColumns}
              onChange={(e) => handleColumnSelect(e)}
            />
            <span>{heading}</span>
          </div>
        )}
      </div>
    );
  };

  return (
    <>
      <div
      data-testid="treegrid-container"
        className={`TreeGrid-container ${
          expanded ? "TreeGrid-container-expanded" : "TreeGrid-container-short"
        }`}
      >
      <div className="TreeGrid-list-container">
        <div className="TreeGrid-list-container-header">
          <span>
            WorkSpace 1 <i className="fas fa-inbox"></i>       
          </span>
        </div>
        <div className="TreeGrid-list-container-body">
        <span>
        <i className="fas fa-search"></i>Search
          </span>
        <span>
        <i className="far fa-bell"></i>Notifications
          </span>
        <span>
        <i className="fas fa-cog"></i>Settings
          </span>
        </div>
      </div>
      <div>
        <div>
        <input
          placeholder="column header"
          value={inputColumn}
          onChange={(e) => setInputColumn(e.target.value)}
        />
        <button onClick={() => addColumn(inputColumn)}>add column</button>          
        </div>
        <button onClick={() => handleDeleteRollup()}>Delete rollup</button>
      </div>
        <div>
          {!expanded ? (
            <button className="TreeGrid-viewbtn" data-testid="expandbutton" onClick={() => handleExpand()}>
              <i className="fas fa-angle-double-right"></i>
            </button>
          ) : (
            <button
              className="TreeGrid-viewbtn"
              onClick={() => handleCollapse()}
              data-testid="collapsebutton"
            >
              <i className="fas fa-angle-double-left"></i>
            </button>
          )}
          <i
            className={`far fa-plus-square fa-lg TreeGrid-menubtn ${
              expanded && "TreeGrid-menubtnexpanded"
            }`}
          ></i>

        </div>
        {data && data.length > 0 && (
          <TreeGridComponent
            dataSource={data}
            allowReordering={"true"}
            childMapping="subtask"
            allowResizing={true}
            treeColumnIndex={0}
            toolbar={toolbarSettings}
            editSettings={editOptions}
            dataBound={autoFit}
            ref={ref}
            queryCellInfo={customizeCell}
          >
            <ColumnsDirective>
              {keys.map((a, i) => {
                return (
                  <ColumnDirective
                    field={a}
                    headerTemplate={() => headerDiv(a, i)}
                    visible={["ID"].includes(a) ? false : true}
                    isPrimaryKey={["ID"].includes(a) ? true : false}
                    lockColumn={["category"].includes(a) ? true : false}
                    allowResizing={["category"].includes(a) ? false : true}
                    allowEditing={["category"].includes(a) ? true : false}
                    key={a}
                    width={"110px"}
                    textAlign="Left"
                  ></ColumnDirective>
                );
                // }
              })}
            </ColumnsDirective>
            <Inject services={[Reorder, Resize, Edit, Toolbar]} />
          </TreeGridComponent>
        )}
      </div>
    </>
  );
};

export default React.memo(TreeGrid);
